/*
 * .vantronix | gvoid11 (gtk+ 802.11b penetration testing utility)
 *
 * Copyright (c) 2002-2004, Reyk Floeter <reyk@vantronix.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"

GtkWidget *dialog_about;

int list_store_cols = 6;
GtkListStore *list_store;
GtkTreeModel *list_model;
GtkTreeView *list_widget;

int running = 0;

void
on_dialog_about_close                  (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(dialog_about);
  gtk_widget_destroy(dialog_about);
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if((dialog_about = create_dialog_about()) != NULL)
    gtk_widget_show (dialog_about);
}


void
on_togglebutton_run_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  int ret = 0;

  if(gtk_toggle_button_get_active(togglebutton)) {
    if(running == 0) {
      gvoid11_setup();

      running = 1;
      if((ret = gvoid11_init()) != 0) {
	gvoid11_warning(strerror(-ret));
	running = 0;
      }
    }
  } else {
#if 0
    gvoid11_warning("Unable to stop execution (NOT YET IMPLEMENTED)\n");
#else
    if(running == 1) {
      running = 0;
      usleep(4 * delay);
      gvoid11_exit();
    }
#endif
  }
}


void
on_treeview_sessions_realize           (GtkWidget       *widget,
                                        gpointer         user_data)
{
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;

   gtk_widget_set_redraw_on_allocate(widget, TRUE);

   list_store = gtk_list_store_new (list_store_cols, 
				    G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
   list_model = GTK_TREE_MODEL(list_store);
   list_widget = GTK_TREE_VIEW(widget);

   gtk_tree_view_set_model(list_widget, list_model);
   renderer = gtk_cell_renderer_text_new();

   column = gtk_tree_view_column_new_with_attributes ("session", renderer,
                                                      "text", GVOID11_COL_SESSION,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_widget), column);

   column = gtk_tree_view_column_new_with_attributes ("channel", renderer,
                                                      "text", GVOID11_COL_CHANNEL,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_widget), column);

   column = gtk_tree_view_column_new_with_attributes ("BSSID", renderer,
                                                      "text", GVOID11_COL_BSSID,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_widget), column);

   column = gtk_tree_view_column_new_with_attributes ("SSID", renderer,
                                                      "text", GVOID11_COL_SSID,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_widget), column);

   column = gtk_tree_view_column_new_with_attributes ("timeout", renderer,
                                                      "text", GVOID11_COL_TIMEOUT,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_widget), column);

   column = gtk_tree_view_column_new_with_attributes ("status", renderer,
                                                      "text", GVOID11_COL_STATUS,
                                                      NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_widget), column);
}


void
on_checkbutton_hopper_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if(gtk_toggle_button_get_active(togglebutton)) {
    /* start new hopper */
    pthread_create(&hopper_thread, NULL, gvoid11_hopper, NULL);
  } else
    pthread_cancel(hopper_thread);
}


void
on_checkbutton_monitor_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
}


void
on_warning_button_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  usleep(delay);

  /* ugly */
  gtk_widget_hide(GTK_WIDGET(button)->parent->parent);
  gtk_widget_destroy(GTK_WIDGET(button)->parent->parent);
}


void
on_spinbutton_debug_changed            (GtkEditable     *editable,
                                        gpointer         user_data)
{
  debug = void11.conf->debug = atoi(gtk_editable_get_chars(editable, 0, 1));
}


void
on_radiobutton_white_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  int i = 0;

  if(gtk_toggle_button_get_active(togglebutton)) 
    void11_match = VOID11_MATCH_WHITE;
  else
    void11_match = VOID11_MATCH_BLACK;

  for(i = 0; i < max_clients; i++) {
    if(gvoid11_iter[i].used < 1) {
      gtk_list_store_set(list_store, &gvoid11_iter[i].iter, 
			 GVOID11_COL_SESSION, "", -1);
      gtk_list_store_set(list_store, &gvoid11_iter[i].iter, 
			 GVOID11_COL_CHANNEL, "", -1);
      gtk_list_store_set(list_store, &gvoid11_iter[i].iter, 
			 GVOID11_COL_BSSID, "", -1);
      gtk_list_store_set(list_store, &gvoid11_iter[i].iter, 
			 GVOID11_COL_SSID, "", -1);
      gtk_list_store_set(list_store, &gvoid11_iter[i].iter, 
			 GVOID11_COL_TIMEOUT, "", -1);
      gtk_list_store_set(list_store, &gvoid11_iter[i].iter, 
			 GVOID11_COL_STATUS, "", -1);
    }
  }
}


void
on_radiobutton_hopper_linear_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if(gtk_toggle_button_get_active(togglebutton)) 
    hopper_mode = GVOID11_HOPPER_LINEAR;
  else
    hopper_mode = GVOID11_HOPPER_RANDOM;
}


void
on_spinbutton_hopper_min_changed       (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget *button;
  button = GTK_WIDGET(editable);
  hopper_start = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(button));
}


void
on_spinbutton_hopper_max_changed       (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget *button;
  button = GTK_WIDGET(editable);
  hopper_stop = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(button));
}


void
on_spinbutton_hopper_delay_changed     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  GtkWidget *button;
  button = GTK_WIDGET(editable);
  hopper_delay = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(button));
}


void
on_readme1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gvoid11_show_readme();
}


void
on_button_readme_close_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(gvoid11_readme);
  gtk_widget_destroy(gvoid11_readme);
}

