/*
 * .vantronix | gvoid11 (gtk+ 802.11b penetration testing utility)
 *
 * Copyright (c) 2002-2004, Reyk Floeter <reyk@vantronix.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */

/* based on hostapd by Jouni Malinen <jkmaline@cc.hut.fi> */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <syslog.h>
#include <time.h>
#include <getopt.h>
#include <signal.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "main.h"

GtkWidget *gvoid11, *gvoid11_readme;

int debug = 0;
int delay = VOID11_DEFAULT_DELAY;
int type = VOID11_DEFAULT_TYPE;
int max_clients = VOID11_MAX_CLIENTS;
int timeout = VOID11_DEFAULT_TIMEOUT;
int void11_match = VOID11_MATCH_WHITE;
int client_connections = 0; /* do not change: initial value */
int match_ssid = 0;
int hopper_mode = GVOID11_HOPPER_RANDOM, 
  hopper_mode, hopper_start, hopper_stop, hopper_delay;
pid_t ppid;
char interface[IFNAMSIZ];
char *staptr = NULL;
char type_str[255];
hostapd void11;

LIST_HEAD(aps_head, entry) aps_black_head, aps_match_head;
struct aps_head *aps_headp;   
struct entry *aps_black, *aps_match;

pthread_t listen_thread, hopper_thread;
static pthread_mutex_t void11_mutex = PTHREAD_MUTEX_INITIALIZER;

struct iter *gvoid11_iter;

static void gvoid11_show_ap(int slot, long timeout, int channel, u8 *bssid, 
			    char *ssid, int ssid_len, char *status)
{
  char i_session[32], i_timeout[32], i_channel[6], i_bssid[32], i_ssid[255], i_status[255];
		    
  snprintf(i_session, sizeof(i_session), "%d", slot + 1);
  snprintf(i_timeout, sizeof(i_timeout), "%ld", timeout);
  snprintf(i_channel, sizeof(i_channel), "%d", channel);
  snprintf(i_bssid, sizeof(i_bssid), MACSTR, MAC2STR(bssid));
  snprintf(i_ssid, sizeof(i_ssid), "%*s", ssid_len, ssid);
  snprintf(i_status, sizeof(i_status), "%s", status);
  
  gtk_list_store_set(list_store, &gvoid11_iter[slot].iter, 
		     GVOID11_COL_SESSION, i_session, -1);
  gtk_list_store_set(list_store, &gvoid11_iter[slot].iter, 
		     GVOID11_COL_CHANNEL, i_channel, -1);
  gtk_list_store_set(list_store, &gvoid11_iter[slot].iter, 
		     GVOID11_COL_BSSID, i_bssid, -1);
  gtk_list_store_set(list_store, &gvoid11_iter[slot].iter, 
		     GVOID11_COL_SSID, i_ssid, -1);
  gtk_list_store_set(list_store, &gvoid11_iter[slot].iter, 
		     GVOID11_COL_TIMEOUT, i_timeout, -1);
  gtk_list_store_set(list_store, &gvoid11_iter[slot].iter, 
		     GVOID11_COL_STATUS, i_status, -1);
}

static int gvoid11_load_match(void)
{
  char *match_ap, str[255], *match_ptr;
  gint lines = 0, i;
  GtkWidget *textview_match;
  GtkTextBuffer *textbuffer;

  if((textview_match = lookup_widget(gvoid11, "textview_match")) == NULL)
    return(-ENOENT);

  textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview_match));
  lines = gtk_text_buffer_get_line_count(textbuffer);

  for(i = 0; i < (lines - 1); i++) {
    struct void11_ap *ptr;
    struct entry *nentry;
    GtkTextIter iter, iter_end;

    gtk_text_buffer_get_iter_at_line(textbuffer, &iter, i);
    gtk_text_buffer_get_iter_at_line(textbuffer, &iter_end, i + 1);

    if((match_ap = (char*)gtk_text_iter_get_text(&iter, &iter_end)) == NULL)
      continue;

    if((ptr = malloc(sizeof(struct void11_ap))) == NULL)
      return(-ENOMEM);
    
    if((nentry = malloc(sizeof(struct entry))) == NULL) {
      free(ptr);
      return(-ENOMEM);
    }

    memset(ptr, 0, sizeof(struct void11_ap));
    memset(nentry, 0, sizeof(struct entry));

    if(strlen(match_ap) < 2) {
      free(ptr);
      free(nentry);
      continue;
    }

    nentry->ap = ptr;

    match_ptr = (char*)match_ap + 2;
    
    switch(match_ap[0]) {
    case 'B':
      if(sscanf(match_ptr, MACSTR,
		(unsigned int*)&ptr->bssid[0],
		(unsigned int*)&ptr->bssid[1],
		(unsigned int*)&ptr->bssid[2],
		(unsigned int*)&ptr->bssid[3],
		(unsigned int*)&ptr->bssid[4],
		(unsigned int*)&ptr->bssid[5]) == ETH_ALEN) {
	DPRINT("added to matchlist: BSSID " MACSTR "\n", 
	       MAC2STR(ptr->bssid));
      } else {
	free(ptr);
	free(nentry);
	continue;
      }
      nentry->match_type = MATCH_BSSID;
      break;
    case 'S':
      strncpy(str, match_ptr, sizeof(str));
      
      if(str[strlen(match_ptr) - 1] == '\n')
	str[strlen(match_ptr) - 1] = '\0';
      else
	str[strlen(match_ptr)] = '\0';
      
      if((ptr->elems.ssid = (char*)malloc(strlen(str) + 1)) == NULL) {
	free(ptr);
	free(nentry);
	continue;
      }
      
      strcpy(nentry->ap->elems.ssid, str);
      nentry->ap->elems.ssid_len = strlen(str);
      
      DPRINT("added to matchlist: SSID '%s'\n", str);
      nentry->match_type = MATCH_SSID;
      break;
    default:
      free(ptr);
      free(nentry);
      continue;
    }
    
    LIST_INSERT_HEAD(&aps_match_head, nentry, entries);
  }
  
  return(0);
}

static int gvoid11_get_action(void)
{
  int ret = VOID11_TYPE_NULL;
  char *text;
  GtkWidget *entry_action;

  if((entry_action = lookup_widget(gvoid11, "entry_action")) == NULL ||
     (text = (char*)gtk_entry_get_text(GTK_ENTRY(entry_action))) == NULL) {
    strncpy(type_str, "no action", sizeof(type_str));
    return(VOID11_TYPE_NULL);
  }

  if(strcmp(text, "no action") == 0)
    ret = VOID11_TYPE_NULL;
  else if(strcmp(text, "deauthentication flood") == 0)
    ret = VOID11_TYPE_DEAUTH;
  else if(strcmp(text, "authentication flood") == 0)
    ret = VOID11_TYPE_AUTH;
  else if(strcmp(text, "association flood") == 0)
    ret = VOID11_TYPE_ASSOC;

  strncpy(type_str, text, sizeof(type_str));

  return(ret);
}

static struct entry *gvoid11_ap_add(struct void11_ap *ap)
{
  int i = 0, slot = -1;
  struct entry *nentry, *np;
  enum { AP_NOT_MATCHED = 0, AP_SSID_MATCHED, AP_BSSID_MATCHED } ap_match = 0;
  
  if(void11.conf->debug > 2)
    DPRINT("add: " MACSTR "\n", MAC2STR(ap->bssid));
  
  /* check matchlist */
  for (np = aps_match_head.lh_first; np != NULL;
       np = np->entries.le_next) {
    switch(np->match_type) {
    case MATCH_BSSID:
      if(memcmp(&np->ap->bssid, &ap->bssid, ETH_ALEN) == 0)
	ap_match = AP_BSSID_MATCHED;
      break;
    case MATCH_SSID:
      if(strcmp(np->ap->elems.ssid, ap->elems.ssid) == 0)
	ap_match = AP_SSID_MATCHED;
      break;
    }
  }

  for(i = 0; i < max_clients; i++) 
    if(slot == -1 && gvoid11_iter[i].used == 0) 
      slot = i;
  
  switch(void11_match) {
  case VOID11_MATCH_WHITE:
    if(ap_match == AP_SSID_MATCHED ||
       ap_match == AP_BSSID_MATCHED) {
      if(void11.conf->debug)
	DPRINT("skipping: accesspoint " MACSTR 
	       " (ssid '%*s' channel %d) is whitelisted\n", 
	       MAC2STR(ap->bssid),
	       ap->elems.ssid_len, ap->elems.ssid, 
	       ap->elems.ds_params[0]);

      if(slot > -1)
	gvoid11_show_ap(slot, 0,
			ap->elems.ds_params[0],
			ap->bssid,
			ap->elems.ssid, ap->elems.ssid_len,
			"skipped (on white list)");

      return(NULL);
    }
    break;
  case VOID11_MATCH_BLACK:
    if(ap_match == AP_NOT_MATCHED) {
      if(void11.conf->debug)
	DPRINT("skipping: accesspoint " MACSTR 
	       " (ssid '%*s' channel %d) is not blacklisted\n", 
	       MAC2STR(ap->bssid),
	       ap->elems.ssid_len, ap->elems.ssid, 
	       ap->elems.ds_params[0]);

      if(slot > -1)
	gvoid11_show_ap(slot, 0,
			ap->elems.ds_params[0],
			ap->bssid,
			ap->elems.ssid, ap->elems.ssid_len,
			"skipped (not on black list)");

      return(NULL);
    }
    break;
  }
  
  /* skip existing accesspoint */
  for (np = aps_black_head.lh_first; np != NULL;
       np = np->entries.le_next) {
    if(memcmp(&np->ap->bssid, &ap->bssid, ETH_ALEN) == 0) {
      if((time(NULL) - np->ap->timestamp) > timeout) {
	np->ap->timestamp = time(NULL);
	if(void11.conf->debug)
	  DPRINT("timeout: " MACSTR " (reset)\n",
		 MAC2STR(np->ap->bssid));
	return(NULL);
      } else 
	return(NULL);
    }
  }

  /* add new accesspoint */
  if((nentry = (struct entry*)malloc(sizeof(struct entry))) == NULL)
    return(NULL);
  
  nentry->ap = ap;
  
  LIST_INSERT_HEAD(&aps_black_head, nentry, entries);
  
  return(nentry);
}

static void *gvoid11_penetration(void *arg)
{
  int ret = 0;
  struct entry *np = (struct entry*)arg;
  char i_seen[32];

  pthread_mutex_lock(&gvoid11_iter[np->slot].mutex);

  while(running == 1 && (time(NULL) - np->ap->timestamp) < timeout) {
    pthread_mutex_lock(&void11_mutex);

    switch(type) {
    case VOID11_TYPE_NULL: /* no action for debugging/scanning */
      break;
    case VOID11_TYPE_AUTH:
      if((ret = void11_auth_req(&void11, np->ap->bssid)) != 0)
	goto out;
      break;
    case VOID11_TYPE_ASSOC:
      if((ret = void11_assoc_req(&void11, np->ap->bssid)) != 0)
	goto out;
      break;
    case VOID11_TYPE_DEAUTH:
    default:
      if((ret = void11_deauth_all_stas(&void11, staptr, np->ap->bssid)) != 0)
	goto out;
      break;
    }

    pthread_mutex_unlock(&void11_mutex);

    memset(&i_seen, 0, sizeof(i_seen));
    snprintf(i_seen, sizeof(i_seen), "%ld", timeout - (time(NULL) - np->ap->timestamp));
    gtk_list_store_set(list_store, &gvoid11_iter[np->slot].iter, 
		       GVOID11_COL_TIMEOUT, i_seen, -1);

    usleep(delay);
  }
  
  goto out_ok;
  
 out:
  pthread_mutex_unlock(&void11_mutex);
  usleep(delay * 2); /* try to sleep a bit longer */

 out_ok:
  if(void11.conf->debug)
    DPRINT("timeout: " MACSTR " (cleanup)\n",
	   MAC2STR(np->ap->bssid));
  
  gtk_list_store_set(list_store, &gvoid11_iter[np->slot].iter, 
		     GVOID11_COL_TIMEOUT, "0", -1);
  gtk_list_store_set(list_store, &gvoid11_iter[np->slot].iter, 
		     GVOID11_COL_STATUS, "timeout", -1);

  if(np != NULL) {
    pthread_mutex_unlock(&gvoid11_iter[np->slot].mutex);
    gvoid11_iter[np->slot].used = 0;
    
    if(np->ap != NULL)
      free(np->ap);
    
    LIST_REMOVE(np, entries);
  }    

  client_connections--;

  pthread_exit(&ret);
}

int gvoid11_randval(int min, int max)
{
  return((arc4random() % (((max) + 1)-(min))) + (min));
}

void *gvoid11_hopper(void *arg)
{
  GtkTextBuffer *text_buffer;
  GtkWidget *entry_hopper_channel, *textview_hopper, 
    *spinbutton_hopper_min, *spinbutton_hopper_max, *spinbutton_hopper_delay;
  int ret = 0, hopper_channel = 0, len = 0;
  char c_channel[6], c_error[1024];

  if((entry_hopper_channel = 
      lookup_widget(gvoid11, "entry_hopper_channel")) == NULL)
    goto out;
  if((textview_hopper = 
      lookup_widget(gvoid11, "textview_hopper")) == NULL)
    goto out;
  if((spinbutton_hopper_min = 
      lookup_widget(gvoid11, "spinbutton_hopper_min")) == NULL)
    goto out;
  if((spinbutton_hopper_max = 
      lookup_widget(gvoid11, "spinbutton_hopper_max")) == NULL)
    goto out;
  if((spinbutton_hopper_delay = 
      lookup_widget(gvoid11, "spinbutton_hopper_delay")) == NULL)
    goto out;

  hopper_start = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_hopper_min));
  hopper_stop = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_hopper_max));
  hopper_delay = 
    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_hopper_delay));

  if((text_buffer = gtk_text_buffer_new(NULL)) == NULL)
    goto out;

  hopper_channel = (hopper_start > hopper_stop ? hopper_stop : hopper_start) - 1;

  for(;;) {
    if(running < 1) {
      usleep(delay);
      continue;
    }

    if(hopper_mode == GVOID11_HOPPER_RANDOM) {
      hopper_channel = 
	gvoid11_randval(hopper_start > hopper_stop ? hopper_stop : hopper_start, 
			hopper_stop < hopper_start ? hopper_start : hopper_stop);
    } else {
      hopper_channel++;

      if(hopper_channel > (hopper_stop < hopper_start ? hopper_start : hopper_stop))
	hopper_channel = (hopper_start > hopper_stop ? hopper_stop : hopper_start);
    }

    pthread_mutex_lock(&void11_mutex);

    if((ret = void11_ioctl_setchannel(&void11, hopper_channel)) != 0) {
      memset(&c_error, 0, sizeof(c_error));
      len = snprintf(c_error, sizeof(c_error), "[%ld] channel %d: %s\n", 
		     time(NULL), hopper_channel, strerror(-ret));
      gtk_text_buffer_set_text(text_buffer, c_error, len);
      gtk_text_view_set_buffer(GTK_TEXT_VIEW(textview_hopper), text_buffer);
    }

    pthread_mutex_unlock(&void11_mutex);

    snprintf(c_channel, sizeof(c_channel), "%d", hopper_channel);
    gtk_entry_set_text(GTK_ENTRY(entry_hopper_channel), c_channel); 

    sleep(hopper_delay);
  }

  g_free(text_buffer);
  gtk_entry_set_text(GTK_ENTRY(entry_hopper_channel), ""); 

 out:
  pthread_exit(&ret);
}

void *gvoid11_listen(void *arg)
{
  struct entry *np;
  GtkTreeIter *iter;
  int ret = 0, i = 0;

  while(running == 1) {
    int slot = -1;
    struct void11_ap *ap;
    pid_t pid;

    if(getpid() != ppid)
      pthread_exit(&ret);

    if((ap = void11_read(&void11)) == NULL)
      continue;

    if(client_connections >= max_clients)
      continue;

    for(i = 0; i < max_clients; i++) 
      if(slot == -1 && gvoid11_iter[i].used == 0) 
	slot = i;

    if(slot == -1)
      continue;

    /* add aps if not withlisted or in specified ssid */
    if((np = gvoid11_ap_add(ap)) == NULL) 
      continue;

    np->slot = slot;
    gvoid11_iter[np->slot].used = 1;

    client_connections++;

    gvoid11_show_ap(np->slot, 
		    timeout - (time(NULL) - np->ap->timestamp),
		    ap->elems.ds_params[0],
		    ap->bssid,
		    ap->elems.ssid, ap->elems.ssid_len,
		    type_str);
    
    /* start new penetrator */
    pthread_create(&np->thread, NULL, gvoid11_penetration, np);
    
    usleep(delay);
  }

  pthread_exit(&ret);
}

void gvoid11_warning(char *message)
{
  GtkWidget *warning, *label_warning;

  if((warning = create_dialog_warning()) != NULL) {
    if((label_warning = lookup_widget(warning, "label_warning")) != NULL)
      gtk_label_set_text(GTK_LABEL(label_warning), message);
    gtk_widget_show(warning);
  }
}

void gvoid11_setup(void)
{
  int i = 0;
  GtkWidget *spinbutton_timeout, *spinbutton_sessions, *entry_interface;

  if((spinbutton_timeout = lookup_widget(gvoid11, "spinbutton_timeout")) != NULL)
    timeout = gtk_spin_button_get_value_as_int((GtkSpinButton*) spinbutton_timeout);

  if((spinbutton_sessions = lookup_widget(gvoid11, "spinbutton_sessions")) != NULL)
    max_clients = gtk_spin_button_get_value_as_int((GtkSpinButton*) spinbutton_sessions);

  if((entry_interface = lookup_widget(gvoid11, "entry_interface")) != NULL) {
    memset(interface, 0, sizeof(interface));
    strncpy(interface, gtk_entry_get_text(GTK_ENTRY(entry_interface)), sizeof(interface));
  }
}

void gvoid11_protect(gboolean protect)
{
  int i = 0;
  GtkWidget *widgets[7], *textview_match;

  widgets[0] = lookup_widget(gvoid11, "combo_action");
  widgets[1] = lookup_widget(gvoid11, "hbox_interface");
  widgets[2] = lookup_widget(gvoid11, "hbox_timeout");
  widgets[3] = lookup_widget(gvoid11, "hbox_sessions");
  widgets[4] = lookup_widget(gvoid11, "hbox_mac");
  widgets[5] = lookup_widget(gvoid11, "hbox_bssid");
  widgets[6] = lookup_widget(gvoid11, "hbox_ssid");

  for(i = 0; i < 7; i++) {
    if(widgets[i]) {
      if(protect) 
	gtk_widget_hide(widgets[i]);
      else
	gtk_widget_show(widgets[i]);
    }
  }

  if((textview_match = lookup_widget(gvoid11, "textview_match")) != NULL) {
    gtk_text_view_set_editable(GTK_TEXT_VIEW(textview_match), 
			       protect ? FALSE : TRUE);
  }

}

void gvoid11_exit(void)
{
  int i = 0;

  pthread_mutex_lock(&void11_mutex);

  /* clear match list */
  while(aps_match_head.lh_first != NULL) {
    if(aps_match_head.lh_first->ap != NULL) {
      if(aps_match_head.lh_first->match_type == MATCH_SSID &&
	 aps_match_head.lh_first->ap->elems.ssid != NULL)
	free(aps_match_head.lh_first->ap->elems.ssid);
      free(aps_match_head.lh_first->ap);
    }
    LIST_REMOVE(aps_match_head.lh_first, entries);
  }
  
  /* clear black list */
  while(aps_black_head.lh_first != NULL) {
    if(aps_black_head.lh_first->ap != NULL)
      free(aps_black_head.lh_first->ap);
    LIST_REMOVE(aps_black_head.lh_first, entries);
  }
  
  void11_exit(&void11);

  if(void11.conf)
    free(void11.conf);

  for(i = 0; i < max_clients; i++) {
    pthread_mutex_destroy(&gvoid11_iter[i].mutex);
    gvoid11_iter[i].used = 0;
  }

  if(gvoid11_iter)
    free(gvoid11_iter);

  gvoid11_protect(FALSE);

  pthread_mutex_unlock(&void11_mutex);
}

int gvoid11_init(void)
{
  int ret = 0, i = 0;

  pthread_mutex_lock(&void11_mutex);

  gtk_list_store_clear(list_store);

  memset(&void11, 0, sizeof(struct hostapd_data));

  if((void11.conf = (struct hostapd_config*)
      malloc(sizeof(struct hostapd_config))) == NULL) {
    pthread_mutex_unlock(&void11_mutex);
    return(-ENOMEM);
  }

  if((gvoid11_iter = calloc(max_clients, sizeof(struct iter))) == NULL) {
    free(void11.conf);
    pthread_mutex_unlock(&void11_mutex);
    return(-ENOMEM);
  }

  for(i = 0; i < max_clients; i++) {
    gtk_list_store_append(list_store, &gvoid11_iter[i].iter);
    pthread_mutex_init(&gvoid11_iter[i].mutex, NULL);
    gvoid11_iter[i].used = 0;
  }

  void11.conf->debug = debug;

  void11.radius = NULL;
  void11.sock = void11.ioctl_sock = -1;
  void11.conf->ssid_len = sizeof(void11.conf->ssid);
  memset(void11.conf->ssid, 0, void11.conf->ssid_len - 1);
  
  LIST_INIT(&aps_black_head);
  LIST_INIT(&aps_match_head);

  type = gvoid11_get_action();

  if((ret = void11_init(&void11, interface)) != 0) {
    pthread_mutex_unlock(&void11_mutex);
    gvoid11_exit();
    return(ret);
  }

  gvoid11_load_match();

  gvoid11_protect(TRUE);

  pthread_mutex_unlock(&void11_mutex);

  /* start scanning server */
  pthread_create(&listen_thread, NULL, gvoid11_listen, NULL);
}

void gvoid11_show_readme(void)
{
  int fd = 0;
  struct stat st_buf;
  GtkTextBuffer *text_buffer;
  GtkWidget *statusbar_readme, *textview_readme;

  gvoid11_readme = create_gvoid11_readme ();

  if((statusbar_readme = lookup_widget(gvoid11_readme, "statusbar_readme")) != NULL) {
    gtk_statusbar_push(GTK_STATUSBAR(statusbar_readme), 0, 
		       VOID11_VERSION " - " VOID11_COPYRIGHT);
  }

  /* ugly, i know... quick hack, will be replaced */
  if((textview_readme = lookup_widget(gvoid11_readme, "textview_readme")) != NULL) {
    if((fd = open(PACKAGE_DATA_DIR "/" PACKAGE "/README", O_RDONLY)) > 0) {
      char *readme_buf;

      if((text_buffer = gtk_text_buffer_new(NULL)) == NULL)
	goto display;

      if(fstat(fd, &st_buf) != 0)
	goto display;

      if((readme_buf = (char*)malloc(st_buf.st_size)) == NULL)
	goto display;

      if(read(fd, readme_buf, st_buf.st_size) == st_buf.st_size) {
	gtk_text_buffer_set_text(text_buffer, readme_buf, st_buf.st_size);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(textview_readme), text_buffer);
      }

      free(readme_buf);

 display:
      close(fd);
    }
  }

  gtk_widget_show (gvoid11_readme);
}

int main (int argc, char *argv[])
{
  int ret = 0;
  GtkWidget *statusbar;

  ppid = getpid();
	
  memset(interface, 0, sizeof(interface));
  strncpy(interface, VOID11_DEFAULT_DEVICE, sizeof(interface));
  
#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  gvoid11 = create_gvoid11 ();

  if((statusbar = lookup_widget(gvoid11, "statusbar")) != NULL) {
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), 0, 
		       VOID11_VERSION " - " VOID11_COPYRIGHT);
  }

  gtk_widget_set_double_buffered(gvoid11, TRUE);
  gtk_widget_set_redraw_on_allocate(gvoid11, TRUE);
  gtk_widget_show (gvoid11);

  gtk_main ();
  return 0;
}

